searchData={"content_type":"text/markdown","items":[{"doc":"Integration with Auth0 to authorize m2m communication","ref":"BridgeEx.Auth0AuthorizationProvider.html","title":"BridgeEx.Auth0AuthorizationProvider","type":"module"},{"doc":"","ref":"BridgeEx.Auth0AuthorizationProvider.html#authorization_headers/2","title":"BridgeEx.Auth0AuthorizationProvider.authorization_headers/2","type":"function"},{"doc":"Preload a set of resources marking them as \"external\" and provide the related getter functions.","ref":"BridgeEx.Extensions.ExternalResources.html","title":"BridgeEx.Extensions.ExternalResources","type":"module"},{"doc":"* `resources` (required): enumerable of resource names (atoms) and their paths (strings).\n    Each path is assumed to be relative to the module directory.","ref":"BridgeEx.Extensions.ExternalResources.html#module-options","title":"Options - BridgeEx.Extensions.ExternalResources","type":"module"},{"doc":"```elixir\ndefmodule MyBridge do\n  use BridgeEx.Extensions.ExternalResources,\n    resources: [\n      my_query: \"queries/query.graphql\",\n      my_mutation: \"mutations/mutation.graphql\"\n    ]\n\n  # it generates the following code:\n\n  @external_resource \"#{__DIR__}/queries/query.graphql\"\n  @external_resource \"#{__DIR__}/mutations/mutation.graphql\"\n\n  @spec my_query() :: String.t()\n  def my_query, do: Map.fetch!(external_resources(), :my_query)\n  @spec my_mutation() :: String.t()\n  def my_mutation, do: Map.fetch!(external_resources(), :my_mutation)\n\n  defp external_resources do\n    %{\n      my_query: \"contents of query.graphql\",\n      my_mutation: \"contents of mutation.graphql\"\n    }\n  end\nend\n```","ref":"BridgeEx.Extensions.ExternalResources.html#module-examples","title":"Examples - BridgeEx.Extensions.ExternalResources","type":"module"},{"doc":"Create a Graphql bridge in the given module.\n\nOnce created, a graphql request can be made via `MyBridge.call(\"my-query\", %{\"variable\": \"var\"})`","ref":"BridgeEx.Graphql.html","title":"BridgeEx.Graphql","type":"module"},{"doc":"* `endpoint`: URL of the remote Graphql endpoint.\n  * `auth0`: enable and configure Auth0 for authentication of requests. Takes the form of `[enabled: false, audience: \"target-audience\"]`.\n  * `encode_variables`: if true, encode the Graphql variables to JSON. Defaults to `false`.\n  * `format_response`: transforms camelCase keys in response to snake_case. Defaults to `false`.\n  * `format_variables`: transforms snake_case variable names to camelCase. Defaults to `false`.\n  * `http_headers`: HTTP headers for the request. Defaults to `%{\"Content-type\": \"application/json\"}`.\n  * `http_options`: HTTP options to be passed to Telepoison. Defaults to `[timeout: 1_000, recv_timeout: 16_000]`.\n  * `log_options`: override global configuration for logging errors. Takes the form of `[log_query_on_error: false, log_response_on_error: false]`.\n  * `max_attempts`: number of times the request will be retried upon failure. Defaults to `1`. ⚠️ Deprecated: use `retry_options` instead.\n  * `decode_keys`: determines how JSON keys in GraphQL responses are decoded. Can be set to `:atoms`, `:strings` or `:existing_atoms`. Currently, the default mode is `:atoms` but will be changed to `:strings` in a future version of this library. You are highly encouraged to set this option to `:strings` to avoid [memory leaks and security concerns](https://hexdocs.pm/jason/Jason.html#decode/2-decoding-keys-to-atoms).\n  * `retry_options`: override configuration regarding retries, namely\n    * `delay`: meaning depends on `timing`\n      * `:constant`: retry ever `delay` ms\n      * `:exponential`: start retrying with `delay` ms\n    * `max_retries`. Defaults to `0`\n    * `policy`: a function that takes an error as input and returns `true`/`false` to indicate whether to retry the error or not. Defaults to \"always retry\" (`fn _ -> true end`).\n    * `timing`: either `:exponential` or `:constant`, indicates how frequently retries are made (e.g. every 1s, in an exponential manner and so on). Defaults to `:exponential`","ref":"BridgeEx.Graphql.html#module-options","title":"Options - BridgeEx.Graphql","type":"module"},{"doc":"```elixir\ndefmodule MyBridge do\n  use BridgeEx.Graphql, endpoint: \"http://my-api.com/graphql\"\nend\n\ndefmodule MyBridge do\n  use BridgeEx.Graphql,\n    endpoint: \"http://my-api.com/graphql\",\n    auth0: [enabled: true, audience: \"target-audience\", client: :target_client]\nend\n\n# If you would like to configure these options to use values evaluated at runtime,\n# you can do so through your application config, e.g.\nconfig :bridge_ex, MyBridge,\n  endpoint: Application.fetch_env!(:my_app, :service_endpoint)\n```","ref":"BridgeEx.Graphql.html#module-examples","title":"Examples - BridgeEx.Graphql","type":"module"},{"doc":"Graphql client for BridgeEx.","ref":"BridgeEx.Graphql.Client.html","title":"BridgeEx.Graphql.Client","type":"module"},{"doc":"Calls a GraphQL endpoint","ref":"BridgeEx.Graphql.Client.html#call/4","title":"BridgeEx.Graphql.Client.call/4","type":"function"},{"doc":"* `url`: URL of the endpoint.\n  * `query`: Graphql query or mutation.\n  * `variables`: variables for Graphql query or mutation.\n  * `opts`: various options.","ref":"BridgeEx.Graphql.Client.html#call/4-parameters","title":"Parameters - BridgeEx.Graphql.Client.call/4","type":"function"},{"doc":"* `options`: extra HTTP options to be passed to Telepoison.\n  * `headers`: extra HTTP headers.\n  * `encode_variables`: whether to JSON encode variables or not.\n  * `decode_keys`: how JSON keys in GraphQL responses are decoded. Can be set to `:strings` (recommended), `:atoms` (discouraged due to [security concerns](https://hexdocs.pm/jason/Jason.html#decode/2-decoding-keys-to-atoms) - currently the default, but will be changed to :strings in a future version) or `:existing_atoms` (safer, but may crash the application if an unexpected key is received)\n  * `retry_options`: configures retry attempts. Takes the form of `[max_retries: 1, timing: :exponential]`\n  * `log_options`: configures logging on errors. Takes the form of `[log_query_on_error: false, log_response_on_error: false]`.","ref":"BridgeEx.Graphql.Client.html#call/4-options","title":"Options - BridgeEx.Graphql.Client.call/4","type":"function"},{"doc":"","ref":"BridgeEx.Graphql.Client.html#t:bridge_response/0","title":"BridgeEx.Graphql.Client.bridge_response/0","type":"type"},{"doc":"Behaviour used to implement a formatter adapter which could be used to\nformat types for both input variables and responses.\n```","ref":"BridgeEx.Graphql.Formatter.Adapter.html","title":"BridgeEx.Graphql.Formatter.Adapter","type":"behaviour"},{"doc":"","ref":"BridgeEx.Graphql.Formatter.Adapter.html#c:format/1","title":"BridgeEx.Graphql.Formatter.Adapter.format/1","type":"callback"},{"doc":"Formatter to format map keys to camelCase.","ref":"BridgeEx.Graphql.Formatter.CamelCase.html","title":"BridgeEx.Graphql.Formatter.CamelCase","type":"module"},{"doc":"","ref":"BridgeEx.Graphql.Formatter.CamelCase.html#format/1","title":"BridgeEx.Graphql.Formatter.CamelCase.format/1","type":"function"},{"doc":"Formatter to format map keys to snake_case.","ref":"BridgeEx.Graphql.Formatter.SnakeCase.html","title":"BridgeEx.Graphql.Formatter.SnakeCase","type":"module"},{"doc":"","ref":"BridgeEx.Graphql.Formatter.SnakeCase.html#format/1","title":"BridgeEx.Graphql.Formatter.SnakeCase.format/1","type":"function"},{"doc":"This defines an adapter that supports GraphQL query documents in their\nconventional (in JS) camelCase notation, while allowing the schema to be\ndefined using conventional (in Elixir) snake_case notation, and\ntransforming the names as needed for lookups, results, and error messages.\n\nFor example, this document:\n\n```\n{\n  myUser: createUser(userId: 2) {\n    firstName\n    lastName\n  }\n}\n```\n\nWould map to an internal schema that used the following names:\n\n* `create_user` instead of `createUser`\n* `user_id` instead of `userId`\n* `first_name` instead of `firstName`\n* `last_name` instead of `lastName`\n\nLikewise, the result of executing this (camelCase) query document against our\n(snake_case) schema would have its names transformed back into camelcase on the\nway out:\n\n```\n%{\n  data: %{\n    \"myUser\" => %{\n      \"firstName\" => \"Joe\",\n      \"lastName\" => \"Black\"\n    }\n  }\n}\n```\n\nNote variables are a client-facing concern (they may be provided as\nparameters), so variable names should match the convention of the query\ndocument (eg, camelCase).","ref":"BridgeEx.Graphql.LanguageConventions.html","title":"BridgeEx.Graphql.LanguageConventions","type":"module"},{"doc":"","ref":"BridgeEx.Graphql.LanguageConventions.html#to_external_name/2","title":"BridgeEx.Graphql.LanguageConventions.to_external_name/2","type":"function"},{"doc":"Converts a camelCase to snake_case\n\niex> to_internal_name(\"test\", :read)\n\"test\"\n\niex> to_internal_name(\"testTTT\", :read)\n\"test_t_t_t\"\n\niex> to_internal_name(\"testTest\", :read)\n\"test_test\"\n\niex> to_internal_name(\"testTest1\", :read)\n\"test_test_1\"\n\niex> to_internal_name(\"testTest11\", :read)\n\"test_test_11\"\n\niex> to_internal_name(\"testTest11Pippo\", :read)\n\"test_test_11_pippo\"\n\niex> to_internal_name(\"camelCase23Snake4344\", :read)\n\"camel_case_23_snake_4344\"","ref":"BridgeEx.Graphql.LanguageConventions.html#to_internal_name/2","title":"BridgeEx.Graphql.LanguageConventions.to_internal_name/2","type":"function"},{"doc":"Utils for handling retrying of functions.","ref":"BridgeEx.Graphql.Retry.html","title":"BridgeEx.Graphql.Retry","type":"module"},{"doc":"","ref":"BridgeEx.Graphql.Retry.html#retry/3","title":"BridgeEx.Graphql.Retry.retry/3","type":"function"},{"doc":"Misc utils for handling Graphql requests/responses.","ref":"BridgeEx.Graphql.Utils.html","title":"BridgeEx.Graphql.Utils","type":"module"},{"doc":"","ref":"BridgeEx.Graphql.Utils.html#decode_http_response/4","title":"BridgeEx.Graphql.Utils.decode_http_response/4","type":"function"},{"doc":"","ref":"BridgeEx.Graphql.Utils.html#parse_response/1","title":"BridgeEx.Graphql.Utils.parse_response/1","type":"function"},{"doc":"","ref":"BridgeEx.Graphql.Utils.html#t:client_response/0","title":"BridgeEx.Graphql.Utils.client_response/0","type":"type"},{"doc":"","ref":"BridgeEx.Graphql.Utils.html#t:graphql_response/0","title":"BridgeEx.Graphql.Utils.graphql_response/0","type":"type"},{"doc":"A simple counter for testing purposes.","ref":"BridgeEx.Utils.Counter.html","title":"BridgeEx.Utils.Counter","type":"module"},{"doc":"","ref":"BridgeEx.Utils.Counter.html#count/1","title":"BridgeEx.Utils.Counter.count/1","type":"function"},{"doc":"","ref":"BridgeEx.Utils.Counter.html#increment/1","title":"BridgeEx.Utils.Counter.increment/1","type":"function"},{"doc":"","ref":"BridgeEx.Utils.Counter.html#start/1","title":"BridgeEx.Utils.Counter.start/1","type":"function"},{"doc":"","ref":"BridgeEx.Utils.Counter.html#stop/1","title":"BridgeEx.Utils.Counter.stop/1","type":"function"},{"doc":"# The MIT License (MIT)\n\nCopyright (c) 2020 Prima.it\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"# BridgeEx\n\n[![Build Status](https://drone-1.prima.it/api/badges/primait/bridge_ex/status.svg)](https://drone-1.prima.it/primait/bridge_ex)\n[![Module Version](https://img.shields.io/hexpm/v/bridge_ex.svg)](https://hex.pm/packages/bridge_ex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/bridge_ex/)\n[![Total Downloads](https://img.shields.io/hexpm/dt/bridge_ex.svg)](https://hex.pm/packages/bridge_ex)\n[![License](https://img.shields.io/hexpm/l/bridge_ex.svg)](https://github.com/primait/auth0_ex/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/primait/auth0_ex.svg)](https://github.com/primait/auth0_ex/commits/master)\n\nA library to build bridges to GraphQL services.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Bridges to Graphql services are defined by `use`ing the `BridgeEx.Graphql` macro as follows:\n\n```elixir\ndefmodule MyApp.SomeServiceBridge do\n  use BridgeEx.Graphql, endpoint: \"http://some_service.example.com\", decode_keys: :strings\n\n  def my_query(%{} = variables) do\n    call(\"a graphql query or mutation\", variables, retry_policy: [max_retries: 1])\n  end\nend\n```\n\nBesides `endpoint` and `decode_keys`, the following parameters can be optionally set when `use`ing `BridgeEx.Graphql`:\n\n- `auth0`\n- `encode_variables`\n- `format_response`\n- `format_variables`\n- `http_headers`\n- `http_options`\n- `log_options`\n- `max_attempts` `⚠ Deprecated in favour of retry_options in call method`\n\nThe option `decode_keys` determines how JSON keys in GraphQL responses are decoded. If you don't provide it, it is set by default to `:atoms`, which is **highly discouraged** since it may raise security concerns (see [\"Decoding keys to atoms\" in Jason documentation](https://hexdocs.pm/jason/Jason.html#decode/2-decoding-keys-to-atoms) for more information). Other decoding modes are `:strings` and `:existing_atoms` which are safer. In a future version, this option will be set by default to `:strings`.\n\nRefer to [the documentation](https://hexdocs.pm/bridge_ex/BridgeEx.Graphql.html) for more details.\n\nIf you need more control on your requests you can use [`BridgeEx.Graphql.Client.call`](https://hexdocs.pm/bridge_ex/BridgeEx.Graphql.Client.html#call/7) directly.\n\nThe library supports preloading queries from external files via the `BridgeEx.Extensions.ExternalResources` optional macro:\n\n```elixir\ndefmodule MyApp.SomeServiceBridge do\n  use BridgeEx.Graphql, endpoint: \"http://some_service.example.com\", decode_keys: :strings\n  use BridgeEx.Extensions.ExternalResources, resources: [my_query: \"my_query.graphql\"]\n\n  def my_query(%{} = variables), do: call(my_query(), variables)\nend\n```\n\n#","ref":"readme.html#usage","title":"Usage - Overview","type":"extras"},{"doc":"The options passed to the `BridgeEx.Graphql` macro are evaluated at compile time. If you need the value of one or more options to be evaluated at runtime (e.g. because they depend on environment variables), you can configure those options for a specific bridge in the application environment.\n\nEach bridge will first try to get its options from the ones passed to `use`. If those are not defined, it will try to get them from the `:bridge_ex, __MODULE__` key in the application environment. If the latter are not present either, it will resort to the default values.\n\nHere's an example:\n\n```elixir\n# config.exs\nconfig :bridge_ex, SomeBridge, endpoint: \"http://some-service/graphql\"\n\n# some_bridge.ex\ndefmodule SomeBridge do\n  use BridgeEx.Graphql\n\n  def my_query(%{} = variables) do\n    # this will call http://some-service/graphql\n    call(\"a graphql query or mutation\", variables)\n  end\nend\n```\n\nNOTE: If you pass the same option both in the application configuration and via `use`, the one passed via `use` (evaluated at compile time) will have the precedence.\n\n#","ref":"readme.html#runtime-options","title":"Runtime options - Overview","type":"extras"},{"doc":"When `call`ing you can provide the following options, some of which override the ones provided at the bridge level:\n\n- `headers` (overrides `http_headers` bridge option)\n- `options` (overrides `http_options` bridge option)\n- `retry_options`\n\n#","ref":"readme.html#call-options","title":"Call options - Overview","type":"extras"},{"doc":"`call` can return one of the following values:\n\n- `{:ok, graphql_response}` on success\n- `{:error, graphql_error}` on graphql error (i.e. 200 status code but `errors` array is not `nil`)\n- `{:error, {:bad_response, status_code}}` on non 200 status code\n- `{:error, {:http_error, reason}}` on http error e.g. `:econnrefused`\n\n#","ref":"readme.html#return-values","title":"Return values - Overview","type":"extras"},{"doc":"By default if `max_attempts` is greater than `1`, the bridge retries every error regardless of its value (⚠ This way is deprecated). This behaviour can be customized by providing the `retry_options` to a `call`.\n`retry_options`: override configuration regarding retries, namely\n\n- `delay`: meaning depends on `timing`\n- `:constant`: retry ever `delay` ms\n- `:exponential`: start retrying with `delay` ms\n- `max_retries`. Defaults to `0`\n- `policy`: a function that takes an error as input and returns `true`/`false` to indicate whether to retry the error or not. Defaults to \"always retry\" (`fn _ -> true end`).\n- `timing`: either `:exponential`or`:constant`, indicates how frequently retries are made (e.g. every 1s, in an exponential manner and so on). Defaults to `:exponential`\n\nA policy example:\n\n```elixir\nretry_policy = fn errors ->\n  case errors do\n    {:bad_response, 400} -> true\n    {:http_error, _reason} -> true\n    [%{message: \"some_error\", extensions: %{code: \"SOME_CODE\"}}] -> true\n    _ -> false\n  end\nend\n\ndefmodule BridgeWithCustomRetry do\n  use BridgeEx.Graphql,\n    endpoint: \"http://some_service.example.com/graphql\", decode_keys: :strings\nend\n\nBridgeWithCustomRetry.call(\"myquery\", %{}, retry_options: [policy: retry_policy, max_retries: 2])\n```\n\n### (Deprecated) Global configuration\n\nThe following configuration parameters can be set globally for all bridges in the app, by setting them inside your `config.exs`:\n\n- `config :bridge_ex, log_options: [log_query_on_error: true, log_response_on_error: false]` to customize logging in your bridges\n\nPlease note that this config has been **deprecated** since it's a footgun for umbrella apps and bad library design in general.\n\n#","ref":"readme.html#customizing-the-retry-options","title":"Customizing the retry options - Overview","type":"extras"},{"doc":"`bridge_ex` supports authentication of machine-to-machine calls via Auth0, through the [prima_auth0_ex](https://github.com/primait/auth0_ex) library.\n\nTo use this feature do the following:\n\n- update your `config.exs` with the necessary config to create API consumers with `prima_auth0_ex`, see [the documentation](https://github.com/primait/auth0_ex#api-consumer)\n- add `:prima_auth0_ex` as a dependency in your mix project\n\nThen configure your bridge with the audience of the target service:\n\n```elixir\nuse BridgeEx.Graphql,\n  endpoint: \"...\",\n  decode_keys: :strings,\n  auth0: [\n    enabled: true,\n    audience: \"target_audience\",\n    # You can optionally specify a client (in case you defined multiple ones)\n    # Otherwise, the :default_client will be used instead\n    # client: :target_client\n  ]\n```\n\nNote that Auth0 integration **must be explicitly enabled for each bridge** where you want it by setting `auth0: [enable: true]`, as per the example above.","ref":"readme.html#authenticating-calls-via-auth0","title":"Authenticating calls via Auth0 - Overview","type":"extras"},{"doc":"As a good practice, if you want to mock your bridge for testing, you _should_ define a behaviour:\n\n```elixir\ndefmodule MyApp.SomeService do\n  @callback my_cool_query(any()) :: {:ok, map()} | {:error, any()}\nend\n```\n\nThen implement it for your bridge:\n\n```elixir\ndefmodule MyApp.SomeServiceBridge do\n  @behaviour MyApp.SomeService\n\n  use BridgeEx.Graphql, endpoint: \"...\"\n  ...\nend\n```\n\nAnd finally implement it again for the mock:\n\n```elixir\ndefmodule MyApp.SomeServiceBridgeMock do\n  @behaviour MyApp.SomeService\n\n  alias BridgeEx.Graphql.Utils\n\n  def my_cool_query(%{} = variables) do\n    File.read!(\"some_mock_file.json\")\n    |> Json.decode!(keys: :atoms)\n    # required to parse data\n    |> Utils.parse_response()\n    # optional, if you want to format the response\n    # |> BridgeEx.Graphql.Formatter.SnakeCase.format()\n  end\nend\n```\n\nYou can now set the right module in your `config/*` directory:\n\n```elixir\nconfig :my_app, :some_service_bridge, MyApp.SomeServiceBridge\n\n# or\n\nconfig :my_app, :some_service_bridge, MyApp.SomeServiceBridgeMock\n```\n\nAnd use it in your app from configuration:\n\n```elixir\n@some_service Application.compile_env!(:my_app, :some_service_bridge)\n\n# ...\n\n@some_service.my_cool_query(%{var: 2})\n```\n\nSee [example](example) directory for an implementation, it also works in `dev` and `test` environments.","ref":"readme.html#testing-your-bridge","title":"Testing your bridge - Overview","type":"extras"},{"doc":"`mix deps.get && mix test`","ref":"readme.html#development","title":"Development - Overview","type":"extras"},{"doc":"The package can be installed by adding `bridge_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bridge_ex, \"~> 2.0.0\"}\n    # only if you want auth0 too\n    # {:prima_auth0_ex, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Overview","type":"extras"},{"doc":"Copyright (c) 2020 Prima.it\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license","title":"Copyright and License - Overview","type":"extras"}]}